# frozen_string_literal: true

require 'open3'

# Module that works with the Devfile standard
module Devfile
  class CliError < StandardError; end

  # Set of services to parse a devfile and output k8s manifests
  class Parser
    FILE_PATH = File.expand_path('./../bin/devfile', File.dirname(__FILE__))

    class << self
      def get_deployment(devfile, name, namespace, labels, annotations, replicas)
        call('deployment', devfile, name, namespace, labels, annotations, replicas)
      end

      def get_service(devfile, name, namespace, labels, annotations)
        call('service', devfile, name, namespace, labels, annotations)
      end

      def get_ingress(devfile, name, namespace, labels, annotations, domain_template, ingress_class)
        call('ingress', devfile, name, namespace, labels, annotations, domain_template, ingress_class)
      end

      def get_pvc(devfile, name, namespace, labels, annotations)
        call('deployment', devfile, name, namespace, labels, annotations)
      end

      def get_all(devfile, name, namespace, labels, annotations, replicas, domain_template, ingress_class)
        call('all', devfile, name, namespace, labels, annotations, replicas, domain_template, ingress_class)
      end

      def flatten(devfile)
        call('flatten', devfile)
      end

      private

      def call(*cmd)
        stdout, stderr, status = Open3.capture3({}, FILE_PATH, *cmd.map(&:to_s))

        raise(CliError, stderr) unless status.success?

        stdout
      end
    end
  end
end
